//-- Main Form -----------------------------------------------------------------
//
//  APixel Color
//
//  Copyright  2003-2007 Alexander Kamburov
//  wise_guybg@yahoo.com
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//------------------------------------------------------------------------------
//
//  Contains code from :
//
//    http://www.swissdelphicenter.ch/torry/showcode.php?id=418
//
//    Subject: Re: How to prevent another instance of same application?
//    Newsgroups: borland.public.delphi.vcl.components.writing
//    Date: 1999/01/12
//
//------------------------------------------------------------------------------
unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, AppEvnts, ExtCtrls, Buttons, XPMan, ImgList, ComCtrls, ToolWin,
  ActnList, Menus;

type
  TfrmMain = class(TForm)
    tmrTick: TTimer;
    lblRValue: TLabel;
    lblGValue: TLabel;
    lblBValue: TLabel;
    lblX: TLabel;
    lblY: TLabel;
    lblDelphiColor: TLabel;
    edtRValue: TEdit;
    edtGValue: TEdit;
    edtBValue: TEdit;
    edtX: TEdit;
    edtY: TEdit;
    edtDelphiColor: TEdit;
    lblHexColor: TLabel;
    edtHexColor: TEdit;
    lblSolidColor: TLabel;
    tbToolbar: TToolBar;
    tbtnOnTop: TToolButton;
    tbtnSep1: TToolButton;
    tbtnAbout: TToolButton;
    pnlZoom: TPanel;
    cbZoom: TComboBox;
    lblZoom: TLabel;
    tbtnExit: TToolButton;
    ilImages: TImageList;
    alActions: TActionList;
    aOnTop: TAction;
    aAbout: TAction;
    aExit: TAction;
    lblInfo: TLabel;
    tbtnSep2: TToolButton;
    XPManifest: TXPManifest;
    sbtnHexCopy: TSpeedButton;
    lblHTMLColor: TLabel;
    edtHTMLColor: TEdit;
    sbtnHTMLCopy: TSpeedButton;
    sbtnDelphiCopy: TSpeedButton;
    shpSolidColor: TShape;
    pmView: TPopupMenu;
    aExportView: TAction;
    dlgSave: TSaveDialog;
    miExportView: TMenuItem;
    aCopyView: TAction;
    Copytoclipboard1: TMenuItem;
    N1: TMenuItem;
    dlgColor: TColorDialog;
    Label1: TLabel;
    Label2: TLabel;
    edtXOld: TEdit;
    edtYOld: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    edtXDiff: TEdit;
    edtYDiff: TEdit;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    pnlSight: TPanel;
    imgView: TImage;
    shpSight: TShape;
    procedure tmrTickTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure aOnTopExecute(Sender: TObject);
    procedure aAboutExecute(Sender: TObject);
    procedure aExitExecute(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure sbtnHexCopyClick(Sender: TObject);
    procedure sbtnHTMLCopyClick(Sender: TObject);
    procedure sbtnDelphiCopyClick(Sender: TObject);
    procedure aExportViewExecute(Sender: TObject);
    procedure aCopyViewExecute(Sender: TObject);
    procedure shpSolidColorMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    //screen shot reference
    FDesktop: TCanvas;
    //processes all messages that are sent and scans for our custom message
    //work with settings
    procedure LoadRegistrySettings;
    procedure SaveRegistrySettings;
    //checks is the SHIFT key is pressed
    function IsShiftDown: Boolean;
    //returns a standard representation of the color returned by IntToHex
    function GetHtmlColor(const AHexValue: string): string;
    //refreshes the user interface
    procedure UpdateData;
    //loads data in the user interface
    procedure LoadData(const APixColor: TColor; const APosition: TPoint);
    //draws the zoomed view of the screen
    procedure DrawView(const ADesktop: TCanvas; const APosition: TPoint);
  public
    procedure DefaultHandler(var Message); override;    
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

uses
  Clipbrd, Registry, AboutForm, ConstUnit;

var
  CustomMessage: Cardinal; //Windows message identifier to talk to APixColor

{
function GetAPixColor: TColor;
var
  Mapping: THandle;
  MapView: PInteger;
begin
  Result := 0;

  Mapping := CreateFileMapping(MAXDWORD, nil, PAGE_READWRITE, 0, SizeOf(Integer),
    'APixColorMMF');
    
  if Mapping = 0 then
    Exit;

  if GetLastError() = ERROR_ALREADY_EXISTS then
  begin
    //test if someone else is using the function at the moment
    //this way we can check if the create/map//unmap/close process is ok 
  end;  

  try
    if SendMessage(HWND_BROADCAST, RegisterWindowMessage('APixColorMessage'),
      0, 0) <> NOERROR then
    begin
      MapView := MapViewOfFile(Mapping, FILE_MAP_ALL_ACCESS, 0, 0, 0);

      if not Assigned(MapView) then
        Exit;

      try
        Result := MapView^;
      finally
        UnmapViewOfFile(MapView);
      end;
    end;  
  finally
    CloseHandle(Mapping);
  end;
end;
}


procedure TfrmMain.DefaultHandler(var Message);
var
  Mapping: THandle;
  MapView: PInteger;
begin
  inherited DefaultHandler(Message);

  if (TMessage(Message).Msg = CustomMessage) then
  begin
    Mapping := OpenFileMapping(FILE_MAP_ALL_ACCESS, false, STR_CUSTOM_MMF);

    if Mapping = 0 then
      Exit;

    try
      MapView := MapViewOfFile(Mapping, FILE_MAP_ALL_ACCESS, 0, 0, 0);

      if not Assigned(MapView) then
        Exit;

      try
        MapView^ := shpSolidColor.Brush.Color;
      finally
        UnmapViewOfFile(MapView);
      end;
    finally
      CloseHandle(Mapping);
    end;
  end;
end;//DefaultHandler

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  DesktopFont := true;
  
  LoadRegistrySettings;

  if cbZoom.ItemIndex = -1 then //just check
    cbZoom.ItemIndex := 0;
  frmMain.Caption := STR_APPNAME;
  Application.Title := STR_APPNAME;

  FDesktop := TCanvas.Create;

  //initialize form
  UpdateData;
end;//FormCreate

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  FDesktop.Free;

  SaveRegistrySettings;
end;//FormDestroy

procedure TfrmMain.LoadRegistrySettings;
var
  Reg: TRegistry;
  LoadSettings: Boolean;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    LoadSettings := Reg.KeyExists(REG_KEY);
    if LoadSettings then
    begin
      Reg.OpenKey(REG_KEY, true);
      if Reg.ReadBool('OnTop') then
        aOnTop.Execute;
      frmMain.Left   := Reg.ReadInteger('WinLeft');
      frmMain.Top    := Reg.ReadInteger('WinTop');
      cbZoom.ItemIndex := Reg.ReadInteger('Zoom');
    end;
  finally
    Reg.Free;
  end;
end;//LoadRegistrySettings

procedure TfrmMain.SaveRegistrySettings;
var
  Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_CURRENT_USER;
    Reg.OpenKey(REG_KEY, true);
    Reg.WriteBool('OnTop', aOnTop.Checked);
    Reg.WriteInteger('WinLeft'  , frmMain.Left  );
    Reg.WriteInteger('WinTop'   , frmMain.Top   );
    Reg.WriteInteger('Zoom', cbZoom.ItemIndex);
  finally
    Reg.Free;
  end;
end;//SaveRegistrySettings

function TfrmMain.IsShiftDown: Boolean;
begin
  Result := (GetKeyState(VK_SHIFT) and $8000 = 0);
end;//IsShiftDown

function TfrmMain.GetHtmlColor(const AHexValue: string): string;
var
  Hex: string;
begin
  Hex := AHexValue;
  if Length(Hex) mod 2 = 1 then
    Hex := '0' + Hex;

  if Length(Hex) = 2 then
    Result := Hex + '0000'
  else
    if Length(Hex) = 4 then
      Result := Hex[3] + Hex[4] + Hex[1] + Hex[2] + '00'
    else
      Result := Hex[5] + Hex[6] + Hex[3] + Hex[4] + Hex[1] + Hex[2];
end;//GetHtmlColor

procedure TfrmMain.UpdateData;
var
  Pos: TPoint;
begin
  GetCursorPos(Pos);
  FDesktop.Handle := GetWindowDC(GetDesktopWindow);

  //update data view
  LoadData(FDesktop.Pixels[Pos.X, Pos.Y], Pos);

  //update image view only if mouse not in
  if (Pos.X < imgView.ClientOrigin.X) or
     (Pos.X > imgView.ClientOrigin.X + imgView.Width) or
     (Pos.Y < imgView.ClientOrigin.Y) or
     (Pos.Y > imgView.ClientOrigin.Y + imgView.Height) then
    DrawView(FDesktop, Pos);
end;//UpdateData

procedure TfrmMain.LoadData(const APixColor: TColor; const APosition: TPoint);
var
  RGBValue: LongInt;
begin
  RGBValue := ColorToRGB(APixColor);
  shpSolidColor.Brush.Color := APixColor;
  edtRValue.Text := IntToStr(GetRValue(RGBValue));
  edtGValue.Text := IntToStr(GetGValue(RGBValue));
  edtBValue.Text := IntToStr(GetBValue(RGBValue));
  edtDelphiColor.Text := ColorToString(APixColor);
  edtHexColor.Text := GetHtmlColor(IntToHex(APixColor, 2));
  edtHTMLColor.Text := '#' + edtHexColor.Text;
  edtX.Text := IntToStr(APosition.X);
  edtY.Text := IntToStr(APosition.Y);
  edtXDiff.Text := IntToStr(Abs(APosition.X - StrToIntDef(edtXOld.Text, 0)));
  edtYDiff.Text := IntToStr(Abs(APosition.Y - StrToIntDef(edtYOld.Text, 0)));
end;//LoadData

procedure TfrmMain.DrawView(const ADesktop: TCanvas; const APosition: TPoint);
var
  RectSide, Margin, SightPos, Zoom: Integer;
  Bitmap: TBitmap;
begin
  //init parameters
  Zoom := 2 * (cbZoom.ItemIndex);
  RectSide := imgView.Width - 1;
  if Zoom > 0 then
    Margin := RectSide div (2 * Zoom)
  else
    Margin := RectSide div 2;

  SightPos := RectSide div 2;

  //set sight
  shpSight.Left := imgView.Left + SightPos;
  shpSight.Top  := imgView.Top  + SightPos;
  shpSight.Width  := Zoom + 2;
  shpSight.Height := Zoom + 2;

  //compose offscreen bitmap and draw it to the image view
  Bitmap := TBitmap.Create;
  try
    Bitmap.Width := imgView.Width;
    Bitmap.Height := imgView.Height;

    //draw background
    Bitmap.Canvas.Brush.Color := clBlack;
    Bitmap.Canvas.Rectangle(0, 0, imgView.Width, imgView.Width);

    //draw view
    Bitmap.Canvas.CopyRect(
      Rect(1, 1, RectSide, RectSide),
      ADesktop,
      Rect(APosition.x - Margin, APosition.y - Margin,
           APosition.x + Margin, APosition.y + Margin));

    //update image
    imgView.Canvas.Draw(0, 0, Bitmap);
  finally
    Bitmap.Free;
  end;
end;//DrawView

procedure TfrmMain.tmrTickTimer(Sender: TObject);
begin
  //check key state
  if IsShiftDown then
    tmrTick.Interval := 100
  else
  begin
    if tmrTick.Interval <> 10 then
    begin
      tmrTick.Interval := 10;
      edtXOld.Text := edtX.Text;
      edtYOld.Text := edtY.Text;      
    end;

    UpdateData;
  end;
end;//tmrTickTimer

procedure TfrmMain.aOnTopExecute(Sender: TObject);
begin
  if tbtnOnTop.Down then
    SetWindowPos(frmMain.Handle, HWND_TOPMOST  , 0, 0, 0, 0,
      SWP_NOMOVE or SWP_NOSIZE or SWP_NOACTIVATE)
  else
    SetWindowPos(frmMain.Handle, HWND_NOTOPMOST, 0, 0, 0, 0,
      SWP_NOMOVE or SWP_NOSIZE or SWP_NOACTIVATE);
end;//aOnTopExecute

procedure TfrmMain.aAboutExecute(Sender: TObject);
begin
  if not Assigned(frmAbout) then
    frmAbout := TfrmAbout.Create(nil);
    
  try
    frmAbout.ShowModal;
  finally
    FreeAndNil(frmAbout);
  end;
end;//aAboutExecute

procedure TfrmMain.aExitExecute(Sender: TObject);
begin
  Close;
end;//aExitExecute

procedure TfrmMain.sbtnHexCopyClick(Sender: TObject);
begin
  Clipboard.AsText := edtHexColor.Text;
end;//sbtnHexCopyClick

procedure TfrmMain.sbtnHTMLCopyClick(Sender: TObject);
begin
  Clipboard.AsText := edtHTMLColor.Text;
end;//sbtnHTMLCopyClick

procedure TfrmMain.sbtnDelphiCopyClick(Sender: TObject);
begin
  Clipboard.AsText := edtDelphiColor.Text;
end;//sbtnDelphiCopyClick

procedure TfrmMain.aExportViewExecute(Sender: TObject);
begin
  if dlgSave.Execute then
  begin
    imgView.Picture.SaveToFile(dlgSave.FileName);
  end;
end;//aExportViewExecute

procedure TfrmMain.aCopyViewExecute(Sender: TObject);
var
  format: Word;
  data: Cardinal;
  palette: HPALETTE;
begin
  imgView.Picture.SaveToClipboardFormat(format, data, palette);
  Clipboard.SetAsHandle(format, data);
end;//aCopyViewExecute

procedure TfrmMain.shpSolidColorMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  dlgColor.Color := shpSolidColor.Brush.Color;
  if dlgColor.Execute then
  begin
    imgView.Canvas.Brush.Color := dlgColor.Color;
    imgView.Canvas.Rectangle(0, 0, imgView.Width, imgView.Width);
    edtXOld.Text := IntToStr(0);
    edtYOld.Text := IntToStr(0);    
    LoadData(dlgColor.Color, Point(0, 0));
  end;
end;//shpSolidColorMouseUp

initialization
  CustomMessage := RegisterWindowMessage(STR_CUSTOM_MESSAGE);

end.
